@@ -556,7 +556,7 @@ void wikidumphtml(TCXSTR *rbuf, TCMAP *cols, const char *buri, int bhl, const ch
   const char *text = tcmapget2(cols, "text");
   if(text && *text != '\0'){
     tcxstrprintf(rbuf, "<div class=\"text\">\n");
-    wikitohtml(rbuf, text, *idbuf != '\0' ? idbuf : NULL, buri, bhl + 1, duri);
+    wikitohtml(rbuf, text, *idbuf != '\0' ? idbuf : NULL, buri, bhl + 1, duri, -1, -1);
     tcxstrcat2(rbuf, "</div>\n");
   }
   const char *com = tcmapget2(cols, "comments");
@@ -598,17 +598,21 @@ void wikidumphtml(TCXSTR *rbuf, TCMAP *cols, const char *buri, int bhl, const ch
 
 
 /* Convert a Wiki string into an HTML string. */
-void wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri, int bhl,
-                const char *duri){
+int wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri, int bhl,
+               const char *duri, int abslen, int absobjnum) {
   assert(rbuf && str && buri && bhl >= 0);
+  if(abslen < 0) abslen = INT_MAX;
+  if(absobjnum < 0) absobjnum = INT_MAX;
   TCLIST *lines = tcstrsplit(str, "\n");
   int lnum = tclistnum(lines);
   int headcnts[HEADLVMAX];
   memset(headcnts, 0, sizeof(headcnts));
+  int txtlen = 0;
+  int objnum = 0;
   int tblcnt = 0;
   int imgcnt = 0;
   int ri = 0;
-  while(ri < lnum){
+  while(ri < lnum && txtlen < abslen){
     int lsiz;
     const char *line = tclistval(lines, ri, &lsiz);
     if(lsiz > 0 && line[lsiz-1] == '\r'){
@@ -639,7 +643,7 @@ void wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri,
           tcxstrprintf(rbuf, "\"");
         }
         tcxstrprintf(rbuf, " class=\"ah%d topic\">", abslv);
-        wikitohtmlinline(rbuf, rp, buri, duri);
+        txtlen += wikitohtmlinline(rbuf, rp, buri, duri);
         tcxstrprintf(rbuf, "</h%d>\n", lv);
         for(int i = lv; i < HEADLVMAX; i++){
           headcnts[lv] = 0;
@@ -678,7 +682,7 @@ void wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri,
           tcxstrprintf(rbuf, "<%s>\n", tag);
         }
         tcxstrcat2(rbuf, "<li>");
-        wikitohtmlinline(rbuf, rp, buri, duri);
+        txtlen += wikitohtmlinline(rbuf, rp, buri, duri);
       }
       while(tclistnum(stack) > 0){
         tcxstrcat2(rbuf, "</li>\n");
@@ -706,7 +710,7 @@ void wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri,
           const char *pv = strchr(rp, sep);
           char *field = pv ? tcmemdup(rp, pv - rp) : tcstrdup(rp);
           tcstrtrim(field);
-          wikitohtmlinline(rbuf, field, buri, duri);
+          txtlen += wikitohtmlinline(rbuf, field, buri, duri);
           tcfree(field);
           tcxstrcat2(rbuf, "</td>\n");
           if(!pv) break;
@@ -729,7 +733,7 @@ void wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri,
         rp = tcstrskipspc(rp + 1);
         if(*rp != '\0'){
           tcxstrcat2(rbuf, "<p>");
-          wikitohtmlinline(rbuf, rp, buri, duri);
+          txtlen += wikitohtmlinline(rbuf, rp, buri, duri);
           tcxstrcat2(rbuf, "</p>\n");
         }
       }
@@ -761,6 +765,7 @@ void wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri,
       for(int i = ri; i < ei; i++){
         const char *rp = tclistval2(lines, i);
         tcxstrprintf(rbuf, "%@\n", rp);
+        txtlen += tcstrcntutf(rp);
       }
       tcxstrcat2(rbuf, "</pre>\n");
       tcxstrdel(sep);
@@ -869,47 +874,56 @@ void wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri,
                      align, align, lv, scale);
         if(anc) tcxstrprintf(rbuf, "<a href=\"%@\">", url);
         if(obj){
-          tcxstrprintf(rbuf, "<object data=\"%@\"", url);
-          const char *type = mimetype(url);
-          if(type) tcxstrprintf(rbuf, " type=\"%@\"", type);
-          if(width > 0) tcxstrprintf(rbuf, " width=\"%d%@\"", width, wratio ? "%" : "");
-          if(height > 0) tcxstrprintf(rbuf, " height=\"%d%@\"", height, hratio ? "%" : "");
-          tcxstrprintf(rbuf, ">");
-          if(params){
-            int pnum = tclistnum(params) - 1;
-            for(int i = 0; i < pnum; i += 2){
-              tcxstrprintf(rbuf, "<param name=\"%@\" value=\"%@\" />",
-                           tclistval2(params, i), tclistval2(params, i + 1));
+          if(objnum < absobjnum) {
+            tcxstrprintf(rbuf, "<object data=\"%@\"", url);
+            const char *type = mimetype(url);
+            if(type) tcxstrprintf(rbuf, " type=\"%@\"", type);
+            if(width > 0) tcxstrprintf(rbuf, " width=\"%d%@\"", width, wratio ? "%" : "");
+            if(height > 0) tcxstrprintf(rbuf, " height=\"%d%@\"", height, hratio ? "%" : "");
+            tcxstrprintf(rbuf, ">");
+            if(params){
+              int pnum = tclistnum(params) - 1;
+              for(int i = 0; i < pnum; i += 2){
+                tcxstrprintf(rbuf, "<param name=\"%@\" value=\"%@\" />",
+                             tclistval2(params, i), tclistval2(params, i + 1));
+              }
             }
+            if(alt){
+              tcxstrprintf(rbuf, "%@", alt);
+            } else {
+              tcxstrprintf(rbuf, "object:%d:%@", imgcnt, name);
+            }
+            tcxstrprintf(rbuf, "</object>");
+            objnum++;
           }
-          if(alt){
-            tcxstrprintf(rbuf, "%@", alt);
-          } else {
-            tcxstrprintf(rbuf, "object:%d:%@", imgcnt, name);
-          }
-          tcxstrprintf(rbuf, "</object>");
         } else if(vdo){
-          tcxstrprintf(rbuf, "<video src=\"%@\"", url);
-          if(width > 0) tcxstrprintf(rbuf, " width=\"%d%s\"", width, wratio ? "%" : "");
-          if(height > 0) tcxstrprintf(rbuf, " height=\"%d%s\"", height, hratio ? "%" : "");
-          tcxstrprintf(rbuf, " preload=\"metadata\"");
-          tcxstrprintf(rbuf, " controls=\"controls\"");
-          if(alt){
-            tcxstrprintf(rbuf, " alt=\"%@\"", alt);
-          } else {
-            tcxstrprintf(rbuf, " alt=\"video:%d:%@\"", imgcnt, name);
+          if(objnum < absobjnum) {
+            tcxstrprintf(rbuf, "<video src=\"%@\"", url);
+            if(width > 0) tcxstrprintf(rbuf, " width=\"%d%s\"", width, wratio ? "%" : "");
+            if(height > 0) tcxstrprintf(rbuf, " height=\"%d%s\"", height, hratio ? "%" : "");
+            tcxstrprintf(rbuf, " preload=\"metadata\"");
+            tcxstrprintf(rbuf, " controls=\"controls\"");
+            if(alt){
+              tcxstrprintf(rbuf, " alt=\"%@\"", alt);
+            } else {
+              tcxstrprintf(rbuf, " alt=\"video:%d:%@\"", imgcnt, name);
+            }
+            tcxstrprintf(rbuf, " />");
+            objnum++;
           }
-          tcxstrprintf(rbuf, " />");
         } else {
-          tcxstrprintf(rbuf, "<img src=\"%@\"", url);
-          if(width > 0) tcxstrprintf(rbuf, " width=\"%d%s\"", width, wratio ? "%" : "");
-          if(height > 0) tcxstrprintf(rbuf, " height=\"%d%s\"", height, hratio ? "%" : "");
-          if(alt){
-            tcxstrprintf(rbuf, " alt=\"%@\"", alt);
-          } else {
-            tcxstrprintf(rbuf, " alt=\"image:%d:%@\"", imgcnt, name);
+          if(objnum < absobjnum) {
+            tcxstrprintf(rbuf, "<img src=\"%@\"", url);
+            if(width > 0) tcxstrprintf(rbuf, " width=\"%d%s\"", width, wratio ? "%" : "");
+            if(height > 0) tcxstrprintf(rbuf, " height=\"%d%s\"", height, hratio ? "%" : "");
+            if(alt){
+              tcxstrprintf(rbuf, " alt=\"%@\"", alt);
+            } else {
+              tcxstrprintf(rbuf, " alt=\"image:%d:%@\"", imgcnt, name);
+            }
+            tcxstrprintf(rbuf, " />");
+            objnum++;
           }
-          tcxstrprintf(rbuf, " />");
         }
         if(anc) tcxstrprintf(rbuf, "</a>");
         tcxstrprintf(rbuf, "</div>\n");
@@ -935,7 +949,7 @@ void wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri,
       line = tcstrskipspc(line);
       if(*line != '\0'){
         tcxstrcat2(rbuf, "<p>");
-        wikitohtmlinline(rbuf, line, buri, duri);
+        txtlen += wikitohtmlinline(rbuf, line, buri, duri);
         tcxstrcat2(rbuf, "</p>\n");
         ri++;
       } else {
@@ -944,12 +958,14 @@ void wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri,
     }
   }
   tclistdel(lines);
+  return txtlen;
 }
 
 
 /* Add an inline Wiki string into HTML. */
-void wikitohtmlinline(TCXSTR *rbuf, const char *line, const char *buri, const char *duri){
+int wikitohtmlinline(TCXSTR *rbuf, const char *line, const char *buri, const char *duri){
   assert(rbuf && line && buri);
+  int txtlen = 0;
   while(*line != '\0'){
     const char *pv;
     if(*line == '['){
@@ -991,55 +1007,56 @@ void wikitohtmlinline(TCXSTR *rbuf, const char *line, const char *buri, const ch
           uri = tcstrskipspc(uri);
           tcxstrprintf(rbuf, "<a href=\"%s?name=%?\">", buri, uri);
         }
-        wikitohtmlinline(rbuf, field, buri, duri);
+        txtlen += wikitohtmlinline(rbuf, field, buri, duri);
         tcxstrprintf(rbuf, "</a>");
         tcfree(field);
         line = pv + 2;
       } else if(tcstrfwm(line, "[*") && (pv = strstr(line + 2, "*]")) != NULL){
         char *field = tcmemdup(line + 2, pv - line - 2);
         tcxstrcat2(rbuf, "<strong>");
-        wikitohtmlinline(rbuf, field, buri, duri);
+        txtlen += wikitohtmlinline(rbuf, field, buri, duri);
         tcxstrcat2(rbuf, "</strong>");
         tcfree(field);
         line = pv + 2;
       } else if(tcstrfwm(line, "[\"") && (pv = strstr(line + 2, "\"]")) != NULL){
         char *field = tcmemdup(line + 2, pv - line - 2);
         tcxstrcat2(rbuf, "<cite>");
-        wikitohtmlinline(rbuf, field, buri, duri);
+        txtlen += wikitohtmlinline(rbuf, field, buri, duri);
         tcxstrcat2(rbuf, "</cite>");
         tcfree(field);
         line = pv + 2;
       } else if(tcstrfwm(line, "[+") && (pv = strstr(line + 2, "+]")) != NULL){
         char *field = tcmemdup(line + 2, pv - line - 2);
         tcxstrcat2(rbuf, "<ins>");
-        wikitohtmlinline(rbuf, field, buri, duri);
+        txtlen += wikitohtmlinline(rbuf, field, buri, duri);
         tcxstrcat2(rbuf, "</ins>");
         tcfree(field);
         line = pv + 2;
       } else if(tcstrfwm(line, "[-") && (pv = strstr(line + 2, "-]")) != NULL){
         char *field = tcmemdup(line + 2, pv - line - 2);
         tcxstrcat2(rbuf, "<del>");
-        wikitohtmlinline(rbuf, field, buri, duri);
+        txtlen += wikitohtmlinline(rbuf, field, buri, duri);
         tcxstrcat2(rbuf, "</del>");
         tcfree(field);
         line = pv + 2;
       } else if(tcstrfwm(line, "[#") && (pv = strstr(line + 2, "#]")) != NULL){
         char *field = tcmemdup(line + 2, pv - line - 2);
         tcxstrcat2(rbuf, "<code>");
-        wikitohtmlinline(rbuf, field, buri, duri);
+        txtlen += wikitohtmlinline(rbuf, field, buri, duri);
         tcxstrcat2(rbuf, "</code>");
         tcfree(field);
         line = pv + 2;
       } else if(tcstrfwm(line, "[$") && (pv = strstr(line + 2, "$]")) != NULL){
         char *field = tcmemdup(line + 2, pv - line - 2);
         tcxstrcat2(rbuf, "<var>");
-        wikitohtmlinline(rbuf, field, buri, duri);
+        txtlen += wikitohtmlinline(rbuf, field, buri, duri);
         tcxstrcat2(rbuf, "</var>");
         tcfree(field);
         line = pv + 2;
       } else if(tcstrfwm(line, "[=") && (pv = strstr(line + 2, "=]")) != NULL){
         char *field = tcmemdup(line + 2, pv - line - 2);
         tcxstrprintf(rbuf, "%@", field);
+        txtlen += tcstrcntutf(field);
         tcfree(field);
         line = pv + 2;
       } else {
@@ -1050,6 +1067,8 @@ void wikitohtmlinline(TCXSTR *rbuf, const char *line, const char *buri, const ch
           case '"': tcxstrcat(rbuf, "&quot;", 6); break;
           default: tcxstrcat(rbuf, line, 1); break;
         }
+        if((*line & 0x80) == 0x00 || (*line & 0xe0) == 0xc0 ||
+           (*line & 0xf0) == 0xe0 || (*line & 0xf8) == 0xf0) txtlen++;
         line++;
       }
     } else {
@@ -1060,9 +1079,12 @@ void wikitohtmlinline(TCXSTR *rbuf, const char *line, const char *buri, const ch
         case '"': tcxstrcat(rbuf, "&quot;", 6); break;
         default: tcxstrcat(rbuf, line, 1); break;
       }
+      if((*line & 0x80) == 0x00 || (*line & 0xe0) == 0xc0 ||
+         (*line & 0xf0) == 0xe0 || (*line & 0xf8) == 0xf0) txtlen++;
       line++;
     }
   }
+  return txtlen;
 }
 
 
