@@ -50,7 +50,7 @@ __COMMON_CLINKAGEBEGIN
  *************************************************************************************************/
 
 
-#define TPVERSION      "0.9.24"
+#define TPVERSION      "0.9.25"
 
 #define TUNEBNUM       131071            // bnum tuning parameter
 #define TUNEAPOW       6                 // apow tuning parameter
@@ -113,17 +113,23 @@ void wikidumphtml(TCXSTR *rbuf, TCMAP *cols, const char *buri, int bhl, const ch
    `id' specifies the ID string of the article.  If it is `NULL', the ID is not expressed.
    `buri' specifies the base URI.
    `bhl' specifies the base header level.
-   `duri' specifie the URI of the data directory. */
-void wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri, int bhl,
-                const char *duri);
+   `duri' specifies the URI of the data directory.
+   `abslen' specifies the maximum length of the abstraction.  If it is negative, no abstraction
+   is performed.
+   `absobjnum' specifies the maximum number of embedded objects in the abstraction.  If it is
+   negative, no limit is set.
+   The return value is the total length of the entity texts except tags. */
+int wikitohtml(TCXSTR *rbuf, const char *str, const char *id, const char *buri, int bhl,
+               const char *duri, int abslen, int absobjnum);
 
 
 /* Add an inline Wiki string into HTML.
    `rbuf' specifies the result buffer.
    `line' specifies the inline Wiki string.
    `buri' specifies the base URI.
-   `duri' specifie the URI of the data directory. */
-void wikitohtmlinline(TCXSTR *rbuf, const char *line, const char *buri, const char *duri);
+   `duri' specifie the URI of the data directory.
+   The return value is the length of the entity text except tags. */
+int wikitohtmlinline(TCXSTR *rbuf, const char *line, const char *buri, const char *duri);
 
 
 /* Simplify a date string.
