@@ -52,7 +52,11 @@ const char *g_mimerule;                  // mime dicision rule
 const char *g_title;                     // site title
 int g_searchnum;                         // number of articles in a search page
 int g_listnum;                           // number of articles in a list page
+int g_listabslen;                        // maximum length of an abstract text
+int g_listabsobjnum;                     // maximum number of objects in an abstract text
 int g_feedlistnum;                       // number of articles in a RSS feed
+int g_feedlistabslen;                    // maximum length of an RSS abstract text
+int g_feedlistabsobjnum;                 // maximum number of objects in an abstract text
 int g_filenum;                           // number of files in a file list page
 int g_sidebarnum;                        // number of items in the side bar
 const char *g_commentmode;               // comment mode
@@ -70,7 +74,8 @@ static void readpasswd(void);
 static bool writepasswd(void);
 static void dosession(TCMPOOL *mpool);
 static void setdberrmsg(TCLIST *emsgs, TCTDB *tdb, const char *msg);
-static void setarthtml(TCMPOOL *mpool, TCMAP *cols, int64_t id, int bhl, bool tiny);
+static void setarthtml(TCMPOOL *mpool, TCMAP *cols, int64_t id, int bhl,
+                       int abslen, int absobjnum, bool tiny);
 static TCLIST *searcharts(TCMPOOL *mpool, TCTDB *tdb, const char *cond, const char *expr,
                           const char *order, int max, int skip, bool ls);
 static void getdaterange(const char *expr, int64_t *lowerp, int64_t *upper);
@@ -182,8 +187,16 @@ static int realmain(int argc, char **argv){
       g_searchnum = tclmax(rp ? tcatoi(rp) : 10, 1);
       rp = tctmplconf(g_tmpl, "listnum");
       g_listnum = tclmax(rp ? tcatoi(rp) : 10, 1);
+      rp = tctmplconf(g_tmpl, "listabslen");
+      g_listabslen = rp ? tcatoi(rp) : 256;
+      rp = tctmplconf(g_tmpl, "listabsobjnum");
+      g_listabsobjnum = rp ? tcatoi(rp) : -1;
       rp = tctmplconf(g_tmpl, "feedlistnum");
       g_feedlistnum = tclmax(rp ? tcatoi(rp) : 10, 1);
+      rp = tctmplconf(g_tmpl, "feedlistabslen");
+      g_feedlistabslen = rp ? tcatoi(rp) : 256;
+      rp = tctmplconf(g_tmpl, "feedlistabsobjnum");
+      g_feedlistabsobjnum = rp ? tcatoi(rp) : -1;
       rp = tctmplconf(g_tmpl, "filenum");
       g_filenum = tclmax(rp ? tcatoi(rp) : 10, 1);
       rp = tctmplconf(g_tmpl, "sidebarnum");
@@ -666,7 +679,7 @@ static void dosession(TCMPOOL *mpool){
               } else if(checkfrozen(cols) && !admin){
                 tclistprintf(emsgs, "The frozen tag is not available by normal users.");
               } else {
-                setarthtml(mpool, cols, p_id, 0, false);
+                setarthtml(mpool, cols, p_id, 0, -1, -1, false);
                 tcmapprintf(vars, "titletip", "[preview]");
                 tcmapput2(vars, "view", "preview");
                 tcmapputmap(vars, "art", cols);
@@ -708,7 +721,7 @@ static void dosession(TCMPOOL *mpool){
         tcmapput2(vars, "view", "edit");
         tcmapput2(vars, "wiki", p_wiki);
       } else {
-        setarthtml(mpool, cols, 0, 0, false);
+        setarthtml(mpool, cols, 0, 0, -1, -1, false);
         tcmapprintf(vars, "titletip", "[preview]");
         tcmapput2(vars, "view", "preview");
         tcmapputmap(vars, "art", cols);
@@ -1003,7 +1016,7 @@ static void dosession(TCMPOOL *mpool){
     }
     TCMAP *cols = tcmpoolpushmap(mpool, dbgetart(tdb, p_id));
     if(cols){
-      setarthtml(mpool, cols, p_id, 0, false);
+      setarthtml(mpool, cols, p_id, 0, -1, -1, false);
       if(checkfrozen(cols) && !admin){
         tcmapput2(cols, "frozen", "true");
       } else if(cancom){
@@ -1050,7 +1063,7 @@ static void dosession(TCMPOOL *mpool){
       int64_t id = tcatoi(tclistval2(res, 0));
       TCMAP *cols = tcmpoolpushmap(mpool, id > 0 ? dbgetart(tdb, id) : NULL);
       if(cols){
-        setarthtml(mpool, cols, id, 0, false);
+        setarthtml(mpool, cols, id, 0, -1, -1, false);
         if(checkfrozen(cols) && !admin){
           tcmapput2(cols, "frozen", "true");
         } else if(cancom){
@@ -1076,7 +1089,7 @@ static void dosession(TCMPOOL *mpool){
         int64_t id = tcatoi(tclistval2(res, i));
         TCMAP *cols = tcmpoolpushmap(mpool, id > 0 ? dbgetart(tdb, id) : NULL);
         if(cols){
-          setarthtml(mpool, cols, id, 1, true);
+          setarthtml(mpool, cols, id, 1, -1, -1, true);
           tclistpushmap(arts, cols);
         }
       }
@@ -1115,7 +1128,7 @@ static void dosession(TCMPOOL *mpool){
       int64_t id = tcatoi(tclistval2(res, i));
       TCMAP *cols = tcmpoolpushmap(mpool, id > 0 ? dbgetart(tdb, id) : NULL);
       if(cols){
-        setarthtml(mpool, cols, id, 1, true);
+        setarthtml(mpool, cols, id, 1, -1, -1, true);
         tclistpushmap(arts, cols);
       }
     }
@@ -1168,6 +1181,9 @@ static void dosession(TCMPOOL *mpool){
       if(tclistnum(arcyears) > 0) tcmapputlist(vars, "arcyears", arcyears);
     }
   } else if(*g_frontpage != '\0' && strcmp(p_act, "timeline")){
+
+
+
     // front view
     if(!auth){
       if(mtime <= p_ifmod){
@@ -1196,7 +1212,7 @@ static void dosession(TCMPOOL *mpool){
     if(id > 0){
       TCMAP *cols = tcmpoolpushmap(mpool, dbgetart(tdb, id));
       if(cols){
-        setarthtml(mpool, cols, id, 0, false);
+        setarthtml(mpool, cols, id, 0, -1, -1, false);
         if(checkfrozen(cols) && !admin) tcmapput2(cols, "frozen", "true");
         tcmapputmap(vars, "art", cols);
       }
@@ -1212,7 +1228,14 @@ static void dosession(TCMPOOL *mpool){
       tcdatestrhttp(mtime, 0, numbuf);
       tcmapput2(vars, "lastmod", numbuf);
     }
-    int max = !strcmp(p_format, "atom") ? g_feedlistnum : g_listnum;
+    int max = g_listnum;
+    int abslen = g_listabslen;
+    int absobjnum = g_listabsobjnum;
+    if(!strcmp(p_format, "atom")) {
+      max = g_feedlistnum;
+      abslen = g_feedlistabslen;
+      absobjnum = g_feedlistabsobjnum;
+    }
     int skip = max * (p_page - 1);
     TCLIST *res = searcharts(mpool, tdb, NULL, NULL, p_order, max + 1, skip, true);
     int rnum = tclistnum(res);
@@ -1224,7 +1247,7 @@ static void dosession(TCMPOOL *mpool){
         int64_t id = tcatoi(tclistval2(res, i));
         TCMAP *cols = tcmpoolpushmap(mpool, id > 0 ? dbgetart(tdb, id) : NULL);
         if(cols){
-          setarthtml(mpool, cols, id, 1, false);
+          setarthtml(mpool, cols, id, 1, abslen, absobjnum, false);
           tclistpushmap(arts, cols);
         }
       }
@@ -1257,14 +1280,18 @@ static void dosession(TCMPOOL *mpool){
   }
   if(g_sidebarnum > 0 && strcmp(p_format, "atom")){
     // side bar
-    TCLIST *res = searcharts(mpool, tdb, NULL, NULL, "cdate", g_sidebarnum, 0, true);
+    TCLIST *res = searcharts(mpool, tdb, NULL, NULL, "cdate", g_sidebarnum + 1, 0, true);
     int rnum = tclistnum(res);
+    if(rnum > g_sidebarnum) {
+      rnum = g_sidebarnum;
+      tcmapput2(vars, "sideartsmore", "");
+    }
     TCLIST *arts = tcmpoollistnew(mpool);
     for(int i = 0; i < rnum; i++){
       int64_t id = tcatoi(tclistval2(res, i));
       TCMAP *cols = tcmpoolpushmap(mpool, id > 0 ? dbgetart(tdb, id) : NULL);
       if(cols){
-        setarthtml(mpool, cols, id, 1, true);
+        setarthtml(mpool, cols, id, 1, -1, -1, true);
         tclistpushmap(arts, cols);
       }
     }
@@ -1422,7 +1449,8 @@ static void setdberrmsg(TCLIST *emsgs, TCTDB *tdb, const char *msg){
 
 
 /* set the HTML data of an article */
-static void setarthtml(TCMPOOL *mpool, TCMAP *cols, int64_t id, int bhl, bool tiny){
+static void setarthtml(TCMPOOL *mpool, TCMAP *cols, int64_t id, int bhl,
+                       int abslen, int absobjnum, bool tiny){
   if(g_scrextproc && scrextcheckfunc(g_scrextproc, "_procart")){
     TCXSTR *wiki = tcmpoolxstrnew(mpool);
     wikidump(wiki, cols);
@@ -1480,8 +1508,12 @@ static void setarthtml(TCMPOOL *mpool, TCMAP *cols, int64_t id, int bhl, bool ti
       tcmapput2(cols, "texttiny", str);
     } else {
       TCXSTR *xstr = tcmpoolxstrnew(mpool);
-      wikitohtml(xstr, rp, idbuf, g_scriptname, bhl + 1, g_uploadpub);
-      if(tcxstrsize(xstr) > 0) tcmapput(cols, "texthtml", 8, tcxstrptr(xstr), tcxstrsize(xstr));
+      int txtlen = wikitohtml(xstr, rp, idbuf, g_scriptname, bhl + 1, g_uploadpub,
+                              abslen, absobjnum);
+      if(tcxstrsize(xstr) > 0) {
+        tcmapput(cols, "texthtml", 8, tcxstrptr(xstr), tcxstrsize(xstr));
+        if(abslen > 0 && txtlen > abslen) tcmapput2(cols, "omitted", "true");
+      }
     }
   }
   rp = tcmapget2(cols, "comments");
